#ifndef NODE_H
#define NODE_H

#include <QGraphicsTextItem>

class Scene;

class Node : public QGraphicsTextItem
{
	Q_OBJECT

public:
	enum {
		PC_SPACE = 16,	//	eqm[hԊu
		CC_SPACE = 4,	//	qm[hԊu
	};
	enum {
		FORK_STYLE = 0,
		RECT_STYLE,
		ROUND_RECT_STYLE,
		CIRCLE_RECT_STYLE,
	};

public:
	Node(const QString &text = QString(), Node *parent = 0);
	~Node();

public:
	bool	expanded() const { return m_expanded; }
	bool	isRootNode() const;
	bool	hasChildren() const { return !m_children.isEmpty(); }
	Node	*parentNode() const;
	Node	*firstChildNode() const;	//	ŏ̎qm[hԂBqǂꍇ 0 Ԃ
	Node	*prevNode() const;		//	Om[hԂBZ̏ꍇ 0 Ԃ
	Node	*nextNode() const;		//	̃m[hԂBq̏ꍇ 0 Ԃ
	qreal	childrenHeight() const;		//	qm[hŜ̍Ԃ
	qreal	treeHeight() const;		//	qm[h܂߂Ŝ̍Ԃ
	int		childIndexOf(Node *) const;
	QRectF	boundingRect() const;
	uchar	nodeStyle() const { return m_nodeStyle; }
	QString	toXmlText() const;
	bool	isDescendantNode(const Node *) const;	//	qm[hǂ`FbN

public:
	void	setExpanded(bool b) { makeChildrenVisible(m_expanded = b); }
	void	makeChildrenVisible(bool b);	//	qm[h̕\\Ԑݒ
	qreal	layoutChildren();		//	qm[hʒu߁Ac[̍Ԃ
	void	removeChildNode(Node *);	//	wm[hqm[hXg폜
	Node	*addNode(Node *);
	void	insert(int, Node *);	//	wԖڂ̎qƂăm[hǉ
	bool	moveUp();
	bool	moveDown();
	bool	moveLeft();
	bool	moveRight();
	bool	levelUp();		//	m[h̊KwɈړ
	bool	levelDown();	//	m[h̊KwɈړ
	Scene	*scene();

protected:
    void    setBranch();            //  }ZbgAbv

protected:
	void	paint(QPainter *, const QStyleOptionGraphicsItem *, QWidget *);
	void	mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * mouseEvent );
	//void	mousePressEvent ( QGraphicsSceneMouseEvent * event );
	void	mouseMoveEvent ( QGraphicsSceneMouseEvent * event );

signals:
	void	doDelete(Node *);

private:
	bool	m_expanded;			//	WJ
	uchar	m_nodeStyle;		//	m[hX^C tH[N/`/...
	QList<Node *>	m_children;		//	qm[hւ̃|C^̃Xg
	class QGraphicsPathItem	*m_branch;		//	}
	//class QGraphicsLineItem	*m_branch;		//	}
};

#endif // NODE_H
